'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = require('fs');
var path = require('path');
var os = require('os');
var destr = _interopDefault(require('destr'));
var flat = _interopDefault(require('flat'));

var RE_KEY_VAL = /^\s*([^=\s]+)\s*=\s*(.*)?\s*$/;
var RE_LINES = /\n|\r|\r\n/;
var defaults = {
  name: '.conf',
  dir: process.cwd(),
  unflatten: true
};

function withDefaults(options) {
  if (typeof options === 'string') {
    options = {
      name: options
    };
  }

  return Object.assign(Object.assign({}, defaults), options);
}
/**
 * Parse rc contents
 */


function parse(contents, unflatten) {
  if ( unflatten === void 0 ) unflatten = true;

  var config = {};
  var lines = contents.split(RE_LINES);

  for (var i = 0, list = lines; i < list.length; i += 1) {
    var line = list[i];

    var match = line.match(RE_KEY_VAL);

    if (!match) {
      continue;
    } // Key


    var key = match[1];

    if (!key || key === '__proto__' || key === 'constructor') {
      continue;
    }

    config[key] = destr(match[2].trim()
    /* val */
    );
  }

  return unflatten ? flat.unflatten(config) : config;
}
/**
 * Parse rc file
 */

function parseFile(path, unflatten) {
  if ( unflatten === void 0 ) unflatten = true;

  if (!fs.existsSync(path)) {
    return {};
  }

  return parse(fs.readFileSync(path, 'utf-8'), unflatten);
}
/**
 * Read rc file
 * @param name Name of rc file (default: '.conf')
 * @param dir Working directory (default: process.cwd())
 */

function read(options) {
  options = withDefaults(options);
  return parseFile(path.resolve(options.dir, options.name), options.unflatten);
}
/**
 * Read rc from user directory
 * @param name Name of rc file (default: '.conf')
 */

function readUser(options) {
  options = withDefaults(options);
  options.dir = os.homedir();
  return read(options);
}
/**
 * Serialize rc config
 * @param config Unflatten config
 */

function serialize(config, unflatten) {
  if ( unflatten === void 0 ) unflatten = true;

  return Object.entries(unflatten ? flat.flatten(config) : config).map(function (ref) {
    var key = ref[0];
    var val = ref[1];

    return (key + "=" + (typeof val === 'string' ? val : JSON.stringify(val)));
  }).join('\n');
}
/**
 * Write rc config
 * @param config Unflatten config
 * @param name Name of rc file (default: '.conf')
 * @param dir Working directory (default: process.cwd())
 */

function write(config, options) {
  options = withDefaults(options);
  fs.writeFileSync(path.resolve(options.dir, options.name), serialize(config, options.unflatten), {
    encoding: 'utf-8'
  });
}
/**
 * Write rc from to user directory
 * @param name Name of rc file (default: '.conf')
 */

function writeUser(config, options) {
  options = withDefaults(options);
  options.dir = os.homedir();
  write(config, options);
}

exports.defaults = defaults;
exports.parse = parse;
exports.parseFile = parseFile;
exports.read = read;
exports.readUser = readUser;
exports.serialize = serialize;
exports.write = write;
exports.writeUser = writeUser;
